use eframe::egui::{self, Response, RichText, Ui, Widget};

use crate::groq::GROQ_CLIENT;

pub struct ModelResponse {
    pub name: String,
    pub message: String,
    pub tokens: i32,
    pub time: f32,
}

impl ModelResponse {
    pub fn new(name: String, message: String, tokens: i32, time: f32) -> Self {
        Self {
            name,
            message,
            tokens,
            time,
        }
    }

    pub async fn chat_completion(&mut self, prompt: String) {
        let response = GROQ_CLIENT.chat_completion(self.name.clone(), prompt).await;
        self.message = response.choices[0].message.content.clone();
        self.tokens = response.usage.total_tokens;
        self.time = response.usage.total_time;
    }
}

impl Widget for &ModelResponse {
    fn ui(self, ui: &mut Ui) -> Response {
        ui.vertical(|ui| {
            egui::Frame::none()
                .inner_margin(8.0)
                .rounding(4.0)
                .fill(egui::Color32::DARK_GRAY)
                .show(ui, |ui| {
                    ui.label(RichText::new(&self.name).strong());

                    ui.horizontal_wrapped(|ui| {
                        ui.horizontal(|ui| {
                            ui.label("Tokens:");
                            ui.label(self.tokens.to_string());
                        });

                        ui.horizontal(|ui| {
                            ui.label("Time:");
                            ui.label(self.time.to_string());
                        });
                    });

                    egui::Frame::none()
                        .inner_margin(8.0)
                        .rounding(4.0)
                        .fill(egui::Color32::WHITE)
                        .show(ui, |ui| {
                            ui.label(
                                egui::RichText::new(&self.message).color(egui::Color32::BLACK),
                            );
                        });
                })
        }).response
    }
}
